searchData={"items":[{"type":"module","title":"ParameterizedTest","doc":"A utility for defining eminently readable parameterized (or example-based) tests.\n\nParameterized tests look like this:\n\n    param_test \"grants free shipping based on the marketing site's stated policy\",\n               \\\"\\\"\\\"\n               | spending_by_category          | coupon      | gets_free_shipping? |\n               | %{shoes: 19_99, pants: 29_99} |             | false               |\n               | %{shoes: 59_99, pants: 49_99} |             | true                |\n               | %{socks: 10_99}               |             | true                |\n               | %{pants: 1_99}                | \"FREE_SHIP\" | true                |\n               \\\"\\\"\\\",\n               %{\n                 spending_by_category: spending_by_category,\n                 coupon: coupon,\n                 gets_free_shipping?: gets_free_shipping?\n               } do\n      shipping_cost = ShippingCalculator.calculate_shipping(spending_by_category, coupon)\n      free_shipping? = shipping_cost == 0\n      assert free_shipping? == gets_free_shipping?\n    end\n\nAlternatively, if you don't like the Markdown table format, you can supply a\nhand-rolled list of parameters to the `param_test` macro, like this:\n\n    param_test \"shipping policy matches the web site\",\n                [\n                  # Items in the parameters list can be either maps...\n                  %{spending_by_category: %{pants: 29_99}, coupon: \"FREE_SHIP\"},\n                  # ...or keyword lists\n                  [spending_by_category: %{shoes: 19_99, pants: 29_99}, coupon: nil]\n                ],\n                %{spending_by_category: spending_by_category, coupon: coupon} do\n      ...\n    end\n\nJust make sure that each item in the parameters list has the same keys.\n\nThe final option is to pass a path to a *file* that contains your test parameters (we currently support `.md`/`.markdown`, `.csv`, and `.tsv` files), like this:\n\n```elixir\nparam_test \"pull test parameters from a file\",\n            \"test/fixtures/params.md\",\n            %{\n              spending_by_category: spending_by_category,\n              coupon: coupon,\n              gets_free_shipping?: gets_free_shipping?\n            } do\n  ...\nend\n```","ref":"ParameterizedTest.html"},{"type":"module","title":"Why parameterized testing? - ParameterizedTest","doc":"Parameterized testing reduces toil associated with writing tests that cover\na wide variety of different example cases. It also localizes the test logic\ninto a single place, so that at a glance you can see how a number of\ndifferent factors affect the behavior of the system under test.\n\nAs a bonus, a table of examples (with their expected results) often\nmatches how the business communicates the requirements of a system,\nboth internally and to customers—for instance, in a table describing\nshipping costs based on how much a customer spends, where they're\nlocated, whether they've bought a promotional product, etc. This means\nparameterized tests can often be initially created by pulling directly from\na requirements document that your product folks provide, and the\nproduct folks can later read the tests (or at least the parameters table)\nif they want to verify the behavior of the system.\n\nSee the README for more information.","ref":"ParameterizedTest.html#module-why-parameterized-testing"},{"type":"macro","title":"ParameterizedTest.param_test/4","doc":"Defines tests that use your parameters or example data.\n\nUse it like:\n\n    param_test \"works as expected\", your_parameters, %{value: from_context, expected_result: expected_result} do\n      assert MyModule.process(from_context) == expected_result\n    end","ref":"ParameterizedTest.html#param_test/4"},{"type":"function","title":"ParameterizedTest.parse_examples/2","doc":"","ref":"ParameterizedTest.html#parse_examples/2"},{"type":"module","title":"ParameterizedTest.CsvParser","doc":"","ref":"ParameterizedTest.CsvParser.html"},{"type":"function","title":"ParameterizedTest.CsvParser.dump_to_iodata/1","doc":"","ref":"ParameterizedTest.CsvParser.html#dump_to_iodata/1"},{"type":"function","title":"ParameterizedTest.CsvParser.dump_to_stream/1","doc":"","ref":"ParameterizedTest.CsvParser.html#dump_to_stream/1"},{"type":"function","title":"ParameterizedTest.CsvParser.parse_enumerable/2","doc":"","ref":"ParameterizedTest.CsvParser.html#parse_enumerable/2"},{"type":"function","title":"ParameterizedTest.CsvParser.parse_stream/2","doc":"","ref":"ParameterizedTest.CsvParser.html#parse_stream/2"},{"type":"function","title":"ParameterizedTest.CsvParser.parse_string/2","doc":"","ref":"ParameterizedTest.CsvParser.html#parse_string/2"},{"type":"function","title":"ParameterizedTest.CsvParser.to_line_stream/1","doc":"","ref":"ParameterizedTest.CsvParser.html#to_line_stream/1"},{"type":"module","title":"ParameterizedTest.Sigil","doc":"Provides a sigil to wrap parsing of the example test tables.\n\nThe `param_test` macro automatically parses Markdown-style example tables\ninto a list of maps for use in the test contexts, so this sigil is never\n*required* to be used. However, it may occasionally be useful to do things\nlike declare a module attribute which is the pre-parsed example table, or to\ninspect how the table will be parsed.\n\nNote that on Elixir v1.14 and earlier, the sigil is `~x` since those versions\ndon't support multi-character or uppercase sigils.","ref":"ParameterizedTest.Sigil.html"},{"type":"function","title":"ParameterizedTest.Sigil.sigil_PARAMS/2","doc":"Provides a sigil for producing example data that you can use in tests.","ref":"ParameterizedTest.Sigil.html#sigil_PARAMS/2"},{"type":"function","title":"Examples - ParameterizedTest.Sigil.sigil_PARAMS/2","doc":"You can have an arbitrary number of columns and rows. Headers are parsed\nas atoms, while the individual cells are parsed as Elixir values.\n\n    iex> ~PARAMS\"\"\"\n    ...>   | plan      | user_permission | can_invite?     |\n    ...>   | :free     | :admin          | true            |\n    ...>   | :free     | :editor         | \"maybe\"         |\n    ...>   | :free     | :view_only      | false           |\n    ...>   | :standard | :admin          | true            |\n    ...>   | :standard | :editor         | \"tuesdays only\" |\n    ...>   | :standard | :view_only      | false           |\n    ...> \"\"\"\n    [\n      %{plan: :free, user_permission: :admin, can_invite?: true},\n      %{plan: :free, user_permission: :editor, can_invite?: \"maybe\"},\n      %{plan: :free, user_permission: :view_only, can_invite?: false},\n      %{plan: :standard, user_permission: :admin, can_invite?: true},\n      %{plan: :standard, user_permission: :editor, can_invite?: \"tuesdays only\"},\n      %{plan: :standard, user_permission: :view_only, can_invite?: false}\n    ]\n\nYou can optionally include separators between the headers and the data.\n\n    iex> ~PARAMS\"\"\"\n    ...>   | plan      | user_permission | can_invite?     |\n    ...>   |-----------|-----------------|-----------------|\n    ...>   | :free     | :admin          | true            |\n    ...>   | :free     | :editor         | \"maybe\"         |\n    ...> \"\"\"\n    [\n      %{plan: :free, user_permission: :admin, can_invite?: true},\n      %{plan: :free, user_permission: :editor, can_invite?: \"maybe\"}\n    ]\n\nYou can pass the output of `~PARAMS` directly to the `param_test` macro:\n\n    param_test \"distinguishes even and odd numbers\",\n               ~PARAMS\"\"\"\n               | even  | odd |\n               | 2     | 1   |\n               | 4     | 3   |\n               | 6     | 5   |\n               \"\"\",\n               %{even: even, odd: odd} do\n      assert rem(even, 2) == 0\n      assert rem(odd, 2) == 1\n    end","ref":"ParameterizedTest.Sigil.html#sigil_PARAMS/2-examples"},{"type":"module","title":"ParameterizedTest.TsvParser","doc":"","ref":"ParameterizedTest.TsvParser.html"},{"type":"function","title":"ParameterizedTest.TsvParser.dump_to_iodata/1","doc":"","ref":"ParameterizedTest.TsvParser.html#dump_to_iodata/1"},{"type":"function","title":"ParameterizedTest.TsvParser.dump_to_stream/1","doc":"","ref":"ParameterizedTest.TsvParser.html#dump_to_stream/1"},{"type":"function","title":"ParameterizedTest.TsvParser.parse_enumerable/2","doc":"","ref":"ParameterizedTest.TsvParser.html#parse_enumerable/2"},{"type":"function","title":"ParameterizedTest.TsvParser.parse_stream/2","doc":"","ref":"ParameterizedTest.TsvParser.html#parse_stream/2"},{"type":"function","title":"ParameterizedTest.TsvParser.parse_string/2","doc":"","ref":"ParameterizedTest.TsvParser.html#parse_string/2"},{"type":"function","title":"ParameterizedTest.TsvParser.to_line_stream/1","doc":"","ref":"ParameterizedTest.TsvParser.html#to_line_stream/1"},{"type":"extras","title":"Changelog","doc":"# Changelog","ref":"changelog.html"},{"type":"extras","title":"v0.1.0 - Changelog","doc":"- Renamed to `ParameterizedTest`, with the accompanying macro `param_test`.\n    (Why not `parameterized_test`? It's longer, harder to spell, and there are a lot of\n    other accepted spellings, including \"parameterised,\" \"parametrized,\" and \"parametrised.\")\n- Added support for hand-rolled lists of parameters, like:\n\n    ```elixir\n    param_test \"shipping policy matches the web site\",\n                [\n                  # Items in the parameters list can be either maps...\n                  %{spending_by_category: %{pants: 29_99}, coupon:   \"FREE_SHIP\"},\n                  # ...or keyword lists\n                  [spending_by_category: %{shoes: 19_99, pants: 29_99}, coupon: nil]\n                ],\n                %{spending_by_category: spending_by_category, coupon: coupon} do\n    ...\n    end\n    ```\n- Added experimental support for populating test parameters from CSV and TSV files.\n  Eventually I'd like to extend this to other sources like Notion documents. \n  (Feedback welcome—just open an issue!)","ref":"changelog.html#v0-1-0"},{"type":"extras","title":"v0.0.1 - Changelog","doc":"Initial release.","ref":"changelog.html#v0-0-1"},{"type":"extras","title":"ParameterizedTest","doc":"# ParameterizedTest\n\n[![Hex.pm](https://img.shields.io/hexpm/v/parameterized_test)](https://hex.pm/packages/parameterized_test) [![Build and Test](https://github.com/s3cur3/parameterized_test/actions/workflows/elixir-build-and-test.yml/badge.svg)](https://github.com/s3cur3/parameterized_test/actions/workflows/elixir-build-and-test.yml) [![Elixir Quality Checks](https://github.com/s3cur3/parameterized_test/actions/workflows/elixir-quality-checks.yml/badge.svg)](https://github.com/s3cur3/parameterized_test/actions/workflows/elixir-quality-checks.yml) [![Code coverage](https://codecov.io/gh/s3cur3/parameterized_test/graph/badge.svg)](https://codecov.io/gh/s3cur3/parameterized_test)\n\nA utility for defining eminently readable parameterized (or example-based) tests in \nElixir's ExUnit, inspired by [example tests in Cucumber](https://cucumber.io/docs/guides/10-minute-tutorial/?lang=java#using-variables-and-examples).","ref":"readme.html"},{"type":"extras","title":"What are parameterized tests? - ParameterizedTest","doc":"Parameterized tests let you define variables along a number of dimensions \nand re-run the same test body (including all `setup`) for each \ncombination of variables.\n\nAn extremely simple (perhaps too simple!) example:\n\n```elixir\nsetup context do\n  # context.permissions gets set by the param_test below\n  permissions = Map.get(context, :permissions, nil)\n  user = AccountsFixtures.user_fixture{permissions: permissions}\n  %{user: user}\nend\n\nparam_test \"users can view the post regardless of permission level\",\n           \"\"\"\n           | permissions |\n           |-------------|\n           | :admin      |\n           | :editor     |\n           | :viewer     |\n           | nil         |\n           \"\"\",\n           %{user: user, permissions: permissions} do\n  assert Posts.can_view?(user), \"User with #{permissions} permissions should be able to view\"\nend\n```\n\nThat test will run 4 times, with the `:permissions` variable from the table \nbeing applied to the test's context each time (and therefore being made\navailable to the `setup` handler). Thus, under the hood this generates\nfour unique tests, equivalent to doing something like this:\n\n```elixir\nsetup context do\n  permissions = Map.get(context, :permissions, nil)\n  user = AccountsFixtures.user_fixture{permissions: permissions}\n  %{user: user}\nend\n\nfor permission <- [:admin, :editor, :viewer, nil] do\n  @permission permission\n\n  @tag permission: @permission\n  test \"users with #{@permission} can view the post\", %{user: user} do\n    assert Posts.can_view?(user)\n  end\nend\n```\n\nOf course, that example only had a single variable, so you could argue\nthe `for` version is cleaner. But `parameterized_test` supports an arbitrary\nnumber of variables, so you can describe complex business rules like\n\"users get free shipping if they spend more than $100, or if they buy\nsocks, or if they have the right coupon code\":\n\n```elixir\nparam_test \"grants free shipping based on the marketing site's stated policy\",\n            \"\"\"\n            | spending_by_category          | coupon      | gets_free_shipping? |\n            |-------------------------------|-------------|---------------------|\n            | %{shoes: 19_99, pants: 29_99} |             | false               |\n            | %{shoes: 59_99, pants: 49_99} |             | true                |\n            | %{socks: 10_99}               |             | true                |\n            | %{pants: 1_99}                | \"FREE_SHIP\" | true                |\n            \"\"\",\n            %{\n               spending_by_category: spending_by_category,\n               coupon: coupon,\n               gets_free_shipping?: gets_free_shipping?\n             } do\n  shipping_cost = ShippingCalculator.calculate_shipping(spending_by_category, coupon)\n  free_shipping? = shipping_cost == 0\n  assert free_shipping? == gets_free_shipping?\nend\n```\n\n## \"I hate the Markdown table syntax!\"\n\nNo sweat, you don't have to use it. You can instead pass a hand-rolled list of\nparameters to the `param_test` macro, like this:\n\n```elixir\nparam_test \"shipping policy matches the web site\",\n            [\n              # Items in the parameters list can be either maps...\n              %{spending_by_category: %{pants: 29_99}, coupon: \"FREE_SHIP\"},\n              # ...or keyword lists\n              [spending_by_category: %{shoes: 19_99, pants: 29_99}, coupon: nil]\n            ],\n            %{spending_by_category: spending_by_category, coupon: coupon} do\n  ...\nend\n```\n\nJust make sure that each item in the parameters list has the same keys.\n\nThe final option is to pass a path to a *file* that contains your test parameters (we currently support `.md`/`.markdown`, `.csv`, and `.tsv` files), like this:\n\n```elixir\nparam_test \"pull test parameters from a file\",\n            \"test/fixtures/params.md\",\n            %{\n              spending_by_category: spending_by_category,\n              coupon: coupon,\n              gets_free_shipping?: gets_free_shipping?\n            } do\n  ...\nend\n```","ref":"readme.html#what-are-parameterized-tests"},{"type":"extras","title":"Why parameterized testing? - ParameterizedTest","doc":"Parameterized testing reduces toil associated with writing tests that cover\na wide variety of different example cases. It also localizes the test logic\ninto a single place, so that at a glance you can see how a number of\ndifferent factors affect the behavior of the system under test.\n\nAs a bonus, a table of examples (with their expected results) often\nmatches how the business communicates the requirements of a system,\nboth internally and to customers—for instance, in a table describing\nshipping costs based on how much a customer spends, where they're\nlocated, whether they've bought a promotional product, etc. This means\nparameterized tests can often be initially created by pulling directly from\na requirements document that your product folks provide, and the\nproduct folks can later read the tests (or at least the parameters table)\nif they want to verify the behavior of the system.","ref":"readme.html#why-parameterized-testing"},{"type":"extras","title":"Parameterized tests versus property tests - ParameterizedTest","doc":"Parameterized tests are superficially similar to property-based tests.\nBoth allow you to write fewer tests while covering more of your system's\nbehavior. This library is not a replacement for property tests, but\nrather complimentary to them.\n\nThere are a few reasons you might choose to write a parameterized\ntest rather than a property test:\n\n- **When describing policies, not invariants**: Much of a system's\n  business logic comes down to arbitrary choices made by a product team.\n  For instance, there's nothing in the abstract description of a shipping\n  calculator that says buying socks or spending $100 total should grant\n  you free shipping. Those aren't *principles* that every correctly\n  implemented shipping system would implement. Instead, they're choices\n  made by someone (maybe a product manager) which will in all likelihood\n  be fiddled with over time.\n  \n  Contrast that with the classic use cases for property tests: every \n  (correct) implementation of, say, `List.sort/1` will *always* have\n  the dual properties of:\n  \n  1. every element of the input being represented in the output, and \n  2. every element being \"less than\" the element after it.\n\n  These sorting properties are *invariants* of the sorting function,\n  and therefore are quite amenable to property testing.\n- **Ease of writing**: Property tests take a lot of practice to get\n  good at writing. They're often quite time consuming to produce, and\n  even when you think you've adequately described the parameters to\n  the system.\n- **For communication with other stakeholders**: The table of examples\n  in a parameterized test can be made readable by non-programmers (or \n  non-Elixir  programmers), so they can be a good way of showing others\n  in your organization which behaviors of the system you've verified.\n  Because they can compactly express a lot of test cases, they're\n  much more suitable for this than saying \"go read the title of\n  every line in this file that starts with `test`.\"\n- **For verifying the exact scenarios described by other stakeholders**:\n  Sometimes the edges of a particular behavior may be fuzzy—not just\n  to you, but in the business domain as well. Hammering out hard-and-fast\n  rules may not be necessary or worth it, so property tests that exercise\n  the boundaries would be overkill. In contrast, when your product\n  folks produce a document that describes the behavior of particular\n  scenarios, you can encode that in a table and ensure that for the\n  cases that are well-specified, the system behaves correctly.\n\nWhen would you write a property test instead of an example tests?\n\n- When you can specify true invariants about the desired behavior\n- When you want the absolute highest confidence in your code\n- When the correctness of a piece of code is important enough to merit\n  a large time investment in getting the tests right\n- When the system's behavior at the edges is well specified\n\nAnd of course there's nothing wrong with using a mix of normal tests,\nparameterized tests, and property tests for a given piece of functionality.","ref":"readme.html#parameterized-tests-versus-property-tests"},{"type":"extras","title":"Installation and writing your first test - ParameterizedTest","doc":"1. Add `parameterized_test` to your `mix.exs` dependencies:\n\n    ```elixir\n    def deps do\n      [\n        {:parameterized_test, \"~> 0.1.0\", only: [:test]},\n      ]\n    end\n    ```\n2. Run `$ mix deps.get` to download the package\n3. Write your first example test by adding `import ParameterizedTest` \n   to the top of your test module, and using the `param_test` macro.\n\n   You can optionally include a separator between the header and body\n   of the table (like `|--------|-------|`).\n\n   The header of your table will be parsed as atoms to pass into your\n   test context. The body cells of the table can be any valid Elixir \n   expression, and empty cells will produce a `nil` value.\n\n   A dummy example:\n\n    ```elixir\n    defmodule MyApp.MyModuleTest do\n      use ExUnit.Case, async: true\n      import ParameterizedTest\n\n      param_test \"behaves as expected\",\n                 \"\"\"\n                 | variable_1       | variable_2           | etc     |\n                 |------------------|----------------------|---------|\n                 | %{foo: :bar}     | div(19, 3)           | false   | \n                 | \"bip bop\"        | String.upcase(\"foo\") | true    |\n                 | [\"whiz\", \"bang\"] | :ok                  |         |\n                 |                  | nil                  | \"maybe\" |\n                 \"\"\",\n                 %{\n                   variable_1: variable_1,\n                   variable_2: variable_2,\n                   etc: etc\n                 } do\n          assert MyModule.valid_combination?(variable_1, variable_2, etc)\n        end\n    end\n    ```","ref":"readme.html#installation-and-writing-your-first-test"},{"type":"extras","title":"About test names - ParameterizedTest","doc":"ExUnit requires each test in a module to have a unique name. To that end,\nParameterizedTest appends a stringified version of the parameters passed\nto your test to the name you give the test. Consider this test:\n\n```elixir\nparam_test \"checks equality\",\n            \"\"\"\n            | val_1 | val_2 |\n            | :a    | :a    |\n            | :b    | :c    |\n            \"\"\",\n            %{val_1: val_1, val_2: val_2} do\n  assert val_1 == val_2\nend\n```\n\nUnder the hood, this produces two tests with the names:\n\n- `\"checks equality (%{val_1: :a, val_b: :a})\"`\n- `\"checks equality (%{val_1: :b, val_b: :c})\"`\n\nAnd if you ran this test, you'd get an error that looks like this:\n\n```\n  1) test checks equality (%{val_1: :b, val_2: :c}) (MyModuleTest)\n     test/my_module_test.exs:4\n     Assertion with == failed\n     code:  assert val_1 == val_2\n     left:  :b\n     right: :c\n     stacktrace:\n       test/my_module_test.exs:11: (test)\n```","ref":"readme.html#about-test-names"}],"content_type":"text/markdown","producer":{"name":"ex_doc","version":[48,46,51,52,46,49]}}